from osv import fields, osv

class hr_contract_status(osv.osv):
	_name = 'acf_custom.contract.status'
	_description = 'Contract status'
	_columns = {
	    'name': fields.char('Status', size=32, required=True),
	    'default': fields.boolean('Is default'),
	}
hr_contract_status()

class hr_contract_type(osv.osv):
	_inherit = 'hr.contract.type'
	_columns = {
	    'management': fields.boolean('Manager contract'),
	    'fixed_term': fields.boolean('Fixed term contract'),
	    'temporary': fields.boolean('Temporary contract'),
	    'part_time': fields.boolean('Part time contract'),
	    'pension': fields.boolean('Contract with pensions'),
	    'pfeligible': fields.boolean('Contract is eligible for PF'),
	    'default': fields.boolean('Is default type'),
	}
hr_contract_type()

class hr_contract(osv.osv):
	_inherit = "hr.contract"
	
	def _get_code(self, cr, uid, ids, field_name, arg=None, context=None):
		return dict((contract.id, contract.employee_id.acf_code + '/' + contract.acf_code) for contract in self.browse(cr, uid, ids, context))
	
	def _get_status(self, cr, uid, context=None):
		status_ids = self.pool.get('acf_custom.contract.status').search(cr, uid, [('default', '=', 'True')])
		return status_ids and status_ids[0] or False
	
	def _get_type(self, cr, uid, context=None):
		type_ids = self.pool.get('hr.contract.type').search(cr, uid, [('default', '=', 'True')])
		return type_ids and type_ids[0] or False
	
	_columns = {
		'acf_contract_status_id': fields.many2one('acf_custom.contract.status', "Status", required=True),
		'acf_code': fields.char('Code', size=64, required=True),
		'name': fields.function(_get_code, type='char', size=256, method=True, string='Name'),
	}
	    
	_defaults = {
		'acf_contract_status_id': _get_status,
		'type_id': _get_type,
		'acf_code': lambda obj, cr, uid, context: obj.pool.get('ir.sequence').get(cr, uid, 'acf_custom.contract')
	}
	_sql_constraints = [
		('acf_code_per_employee_uniq', 'UNIQUE (acf_code, employee_id)', 'The Code must be unique for an employee!')
	    ]

hr_contract() 
