
from osv import fields, osv

class hr_payslip_worked_days(osv.osv):
	_inherit = 'hr.payslip.worked_days'
	_sql_constraints = [
		('wd_code_per_payslip_contract_uniq', 'UNIQUE (payslip_id, contract_id, code)', 'The Worked Days code must be unique per Payslip and Contract !')
	    ]

hr_payslip_worked_days()


class hr_payslip_input(osv.osv):
	_inherit = 'hr.payslip.input'
	_sql_constraints = [
		('input_code_per_payslip_contract_uniq', 'UNIQUE (payslip_id, contract_id, code)', 'The Input code must be unique per Payslip and Contract !')
	    ]

hr_payslip_worked_days()


class hr_salary_rule(osv.osv):
    _inherit = 'hr.salary.rule'
    _order = "sequence asc, id asc"

    #TODO: remove once fixed in OpenERP https://bugs.launchpad.net/openobject-addons/+bug/1044611
    # This fixes three problems: 
    #   1) result_qty and result_rate remaining in the localdict - affecting next calculation
    #   2) result_qty and result_rate set to 1, resp 100 when 0
    #   3) lack of information about failures
    def compute_rule(self, cr, uid, rule_id, localdict, context=None):
        """
        :param rule_id: id of rule to compute
        :param localdict: dictionary containing the environement in which to compute the rule
        :return: returns a tuple build as the base/amount computed, the quantity and the rate 
        :rtype: (float, float, float)
        """
        
	from tools.translate import _
	from tools.safe_eval import safe_eval as eval

	# Make sure no residuals are left behind
	localdict.pop('result', False)
	localdict.pop('result_qty', False)
	localdict.pop('result_rate', False)
	
        rule = self.browse(cr, uid, rule_id, context=context)
        if rule.amount_select == 'fix':
            try:
                return rule.amount_fix, eval(rule.quantity, localdict), 100.0
            except Exception as x:
                raise osv.except_osv(_('Error'), _('Failed calculating quantity defined for salary rule %s (%s), Exception: %s')% (rule.name, rule.code, x))
        elif rule.amount_select == 'percentage':
            try:
                return eval(rule.amount_percentage_base, localdict), eval(rule.quantity, localdict), rule.amount_percentage
            except Exception as x:
                raise osv.except_osv(_('Error'), _('Failed calculating percentage base or quantity defined for salary rule %s (%s), Exception: %s')% (rule.name, rule.code, x))
        else:
            try:
                eval(rule.amount_python_compute, localdict, mode='exec', nocopy=True)
                # Using pop with default to both remove the item and get a default sensibly
                return localdict.pop('result'), localdict.pop('result_qty', 1.0), localdict.pop('result_rate', 100.0)
            except Exception as x:
                raise osv.except_osv(_('Error'), _('Failed calculating python code defined for salary rule %s (%s), Exception: %s')% (rule.name, rule.code, x))

    def satisfy_condition(self, cr, uid, rule_id, localdict, context=None):
        """
        @param rule_id: id of hr.salary.rule to be tested
        @param contract_id: id of hr.contract to be tested
        @return: returns True if the given rule match the condition for the given contract. Return False otherwise.
        """
        
	from tools.translate import _
	from tools.safe_eval import safe_eval as eval

	# Make sure no residuals are left behind
	localdict.pop('result', False)
	
        rule = self.browse(cr, uid, rule_id, context=context)

        if rule.condition_select == 'none':
            return True
        elif rule.condition_select == 'range':
            try:
                result = eval(rule.condition_range, localdict)
                return rule.condition_range_min <=  result and result <= rule.condition_range_max or False
            except Exception as x:
                raise osv.except_osv(_('Error'), _('Failed satisfying range condition defined for salary rule %s (%s), Exception: %s')% (rule.name, rule.code, x))
        else: #python code
            try:
                eval(rule.condition_python, localdict, mode='exec', nocopy=True)
                return localdict.pop('result', False)
            except Exception as x:
                raise osv.except_osv(_('Error'), _('Failed satisfying python condition defined for salary rule %s (%s), Exception: %s')% (rule.name, rule.code, x))
	      
hr_salary_rule()


class hr_payslip(osv.osv):
    _inherit = 'hr.payslip'
    
    # Override the method collecting inputs for payslip calculation so that we can:
    # 1) make sure only unique code lines enter
    # 2) assign an amount to the input based on payslip template
    # 3) include some handy values from related objects, like the contract.wage
    def get_inputs(self, cr, uid, contract_ids, date_from, date_to, context=None):
	
	# First get the inputs expected by salary rules from parent implementation 
	inputs = super(hr_payslip, self).get_inputs(cr, uid, contract_ids, date_from, date_to, context)

	# using a dict to index inputs, but more importantly
	# to ensure unique code - contrct_id key
	inpdict = dict( ((i['code'], i['contract_id']), i) for i in inputs )
	
        contract_model = self.pool.get('hr.contract')
        
        for contract in contract_model.browse(cr, uid, contract_ids, context=context):
	  	  
	    if contract.payslip_template_id:
		for input_default in contract.payslip_template_id.input_default_ids:
		    key = (input_default.code, contract.id)
		    if key in inpdict:
			inpdict[key]['amount'] = input_default.amount
		
	    key = ('SalaryBase', contract.id)
	    if key in inpdict:
		inpdict[key]['amount'] = contract.wage
		
	    key = ('ContractIsTemporary', contract.id)
	    if key in inpdict:
		inpdict[key]['amount'] = int(contract.type_id.temporary)
		
	    key = ('ContractIsManagement', contract.id)
	    if key in inpdict:
		inpdict[key]['amount'] = int(contract.type_id.management)
		
	    key = ('ContractHasPension', contract.id)
	    if key in inpdict:
		inpdict[key]['amount'] = int(contract.type_id.pension)
		
	    key = ('EmployeeIsPFEligible', contract.id)
	    if key in inpdict:
		inpdict[key]['amount'] = int(contract.type_id.pfeligible)
			
	    for input_override in contract.payslip_template_override_ids:
		key = (input_override.code, contract.id)
		if key in inpdict:
		    inpdict[key]['amount'] = input_override.amount
		    
        inputs = inpdict.values()
	  
	return inputs

hr_payslip()


class hr_payslip_template(osv.osv):
  	_name = 'acf_custom.payslip.template'
	_description = 'Payslip template'
	_columns = {
	    'name': fields.char('Name', size=256, required=True),
	    'contract_ids': fields.one2many('hr.contract', 'payslip_template_id', 'Related contracts' ),
	    'input_default_ids': fields.one2many('acf_custom.payslip.template.input.default', 'template_id', 'Related input defaults' ),
	    'input_override_ids': fields.related(
						      'contract_ids',
						      'payslip_template_override_ids',
						      type="one2many",
						      relation="acf_custom.payslip.template.input.override",
						      string="Contract overrides",
						      store=False),
	    'sequence': fields.integer('Sequence', required=True, select=True),
	}
	_order = "sequence asc, id asc"
	_defaults = {
		'sequence' : 0,
	    }

hr_payslip_template()


class acf_custom_payslip_template_input_override(osv.osv):
	_name = 'acf_custom.payslip.template.input.override'
	_description = 'Payslip template input override'
	_columns = {
	    'contract_id': fields.many2one('hr.contract', 'Contract', required=True, ondelete='cascade', select=True, help="The contract for which applied this input override"),
	    'name': fields.char('Description', size=256, required=True),
	    'code': fields.char('Code', size=52, required=True, help="The code that can be used in the salary rules"),
	    'amount': fields.float('Amount', help="It is used in computation. For e.g. A rule for sales having 1% commission of basic salary for per product can defined in expression like result = inputs.SALEURO.amount * contract.wage*0.01."),
	    'sequence': fields.integer('Sequence', required=True, select=True),
        }
	_order = "sequence asc, id asc"
	_sql_constraints = [
		('input_code_per_templte_unique', 'UNIQUE (code, contract_id)', 'The Override input code must be unique per Contract!')
	    ]
	_defaults = {
		'sequence' : 0,
	    }

acf_custom_payslip_template_input_override()


class acf_custom_payslip_template_input_default(osv.osv):
	_name = 'acf_custom.payslip.template.input.default'
	_description = 'Payslip template input default'
	_columns = {
	    'template_id': fields.many2one('acf_custom.payslip.template', 'Template', required=True, ondelete='cascade', select=True),
	    'name': fields.char('Description', size=256, required=True),
	    'code': fields.char('Code', size=52, required=True, help="The code that can be used in the salary rules"),
	    'amount': fields.float('Amount', help="It is used in computation. For e.g. A rule for sales having 1% commission of basic salary for per product can defined in expression like result = inputs.SALEURO.amount * contract.wage*0.01."),
	    'sequence': fields.integer('Sequence', required=True, select=True),
        }
	_order = "sequence asc, id asc"
	_sql_constraints = [
		('input_code_per_templte_unique', 'UNIQUE (code, template_id)', 'The Default input code must be unique per Template and Contract!')
	    ]
	_defaults = {
		'sequence' : 0,
	    }
	    
acf_custom_payslip_template_input_default()


class hr_contract(osv.osv):
	_inherit = "hr.contract"
	
	def get_expected_payslip_input_ids(self, cr, uid, ids, field_names, arg=None, context=None):

	    contract_obj = self.pool.get('hr.contract')
	    rule_obj = self.pool.get('hr.salary.rule')
	    structure_obj = self.pool.get('hr.payroll.structure')

	    if not ids or not len(ids) or not ids[0]:
		return {}
	    
	    structure_ids = contract_obj.get_all_structures(cr, uid, ids, context=context)
	    rule_ids = structure_obj.get_all_rules(cr, uid, structure_ids, context=context)
	    
	    # this might not be quite necessary, but is included for consistency with payslip.get_inputs()
	    sorted_rule_ids = [id for id, sequence in sorted(rule_ids, key=lambda x:x[1])]
	    
	    rules_iter = rule_obj.browse(cr, uid, sorted_rule_ids, context=context)
	    
	    inputs = dict((input.code, input.id) 
			    for rule in rules_iter for input in rule.input_ids )
	    
	    res = dict( (contract_id, inputs.values())  for contract_id in ids)
		
	    return res
	    

	# fixes https://bugs.launchpad.net/acf-openerp-custom-main/+bug/1061213
	#  - Displaying a Contract fails when Salary Structure is not set
	def get_all_structures(self, cr, uid, contract_ids, context=None):
	    """
	    @param contract_ids: list of contracts
	    @return: the structures linked to the given contracts, ordered by hierachy (parent=False first, then first level children and so on) and without duplicata
	    """
	    all_structures = []
	    structure_ids = [contract.struct_id.id for contract in self.browse(cr, uid, contract_ids, context=context)]
	    
	    if not structure_ids or not len(structure_ids) or not structure_ids[0]:
		return []
	    
	    return list(set(self.pool.get('hr.payroll.structure')._get_parent_structure(cr, uid, structure_ids, context=context)))	  
	    
	    
	def onchange_struct_id(self, cr, uid, ids, struct_id, context=None):
	    
	    res = {'value':{'payslip_rule_input_ids':[]}}
	    
	    if not struct_id:
		return res
		
	    rule_obj = self.pool.get('hr.salary.rule')
	    structure_obj = self.pool.get('hr.payroll.structure')
	    
	    # TODO: move these lines into a method of structure_obj
	    structure_ids = structure_obj._get_parent_structure(cr, uid, [struct_id], context=context)
	    rule_ids = structure_obj.get_all_rules(cr, uid, structure_ids, context=context)
	    
	    # this might not be quite necessary, but is included for consistency with payslip.get_inputs()
	    sorted_rule_ids = [id for id, sequence in sorted(rule_ids, key=lambda x:x[1])]
	    
	    rules_iter = rule_obj.browse(cr, uid, sorted_rule_ids, context=context)
	    
	    res['value']['payslip_rule_input_ids'] = [input.id for rule in rules_iter for input in rule.input_ids]
	    
	    return res
	  
	def onchange_payslip_template_id(self, cr, uid, ids, payslip_template_id, context=None):
	  
	    res = {'value':{'payslip_template_default_ids':[]}}
	    
	    if not payslip_template_id:
		return res
	    	  
	    defaults_obj = self.pool.get('acf_custom.payslip.template.input.default')
	    def_ids = defaults_obj.search(cr, uid, [('template_id','=',payslip_template_id)], context=context)
	    
	    res['value']['payslip_template_default_ids'] = def_ids
	    
	    return res

	_columns = {
		'payslip_template_id': fields.many2one('acf_custom.payslip.template', "Payslip template", required=False),
		'payslip_rule_input_ids': fields.function(
						      get_expected_payslip_input_ids,
						      method=True,
						      type="one2many",
						      relation="hr.rule.input",
						      string="Payslip inputs expected",
						      store=False),
		'payslip_template_default_ids': fields.related(
						      'payslip_template_id',
						      'input_default_ids',
						      type="one2many",
						      relation="acf_custom.payslip.template.input.default",
						      string="Payslip template input defaults",
						      store=False),
		'payslip_template_override_ids': fields.one2many(
						      'acf_custom.payslip.template.input.override',
						      'contract_id',
						      "Payslip template input overrides"),
	      }
	    
hr_contract() 